package com.biganto.visual.roompark.data.repository.api.retrofit.di

import android.app.Application
import com.biganto.visual.roompark.data.repository.api.retrofit.IRoomParkMobileApi
import com.biganto.visual.roompark.data.repository.api.retrofit.raw.ErrorRaw
import com.biganto.visual.roompark.data.repository.api.retrofit.util.*
import com.biganto.visual.roompark.data.service.network.INetworkMonitor
import com.biganto.visual.roompark.data.service.network.LiveNetworkMonitor
import com.biganto.visual.roompark.data.service.network.NoNetworkException
import com.biganto.visual.roompark.data.service.version_control.AppVersionManager
import com.biganto.visual.roompark.data.service.version_control.IAppVersionControl
import com.google.gson.GsonBuilder
import com.google.gson.reflect.TypeToken
import dagger.Module
import dagger.Provides
import okhttp3.OkHttpClient
import okhttp3.logging.HttpLoggingInterceptor
import retrofit2.Retrofit
import retrofit2.adapter.rxjava2.RxJava2CallAdapterFactory
import retrofit2.converter.gson.GsonConverterFactory
import retrofit2.converter.scalars.ScalarsConverterFactory
import timber.log.Timber
import java.util.*
import java.util.concurrent.TimeUnit
import javax.inject.Singleton

/**
 * Created by Vladislav Bogdashkin on 29.10.2019.
 */



private const val TIMEOUT_SECONDS=120L
private const val WRITE_SECONDS=120L
private const val READ_SECONDS=120L

val INTERCEPT_LOG_LEVEL = HttpLoggingInterceptor.Level.HEADERS

@Module
class RetrofitModule{

//    var networkMonitor = LiveNetworkMonitor(context)
//
//    var versionControl = AppVersionManager(context)

    init {
        Timber.d("Module inited")
    }

    private inline fun <reified T> genericType() = object : TypeToken<T>() {}.type

    private fun gsonConverterFactory() = GsonConverterFactory.create(
        GsonBuilder()
            .registerTypeHierarchyAdapter(Any::class.java,CustomExceptionDeserializer<Any>())
            .registerTypeAdapter(genericType<MutableList<ErrorRaw>>(), ErrorsListDeserializer())
            .registerTypeAdapter(Date::class.java, DateSerializer())
            .setLenient()
            .create())

    private fun client(appVersionManager: IAppVersionControl, networkMonitor: INetworkMonitor) =
        OkHttpClient.Builder()
            .hostnameVerifier { hostname, session ->  true}
            .connectTimeout(TIMEOUT_SECONDS, TimeUnit.SECONDS)
            .writeTimeout(WRITE_SECONDS, TimeUnit.SECONDS)
            .readTimeout(READ_SECONDS, TimeUnit.SECONDS)
            .addInterceptor {
                if (networkMonitor.isConnected) {
                    it.proceed(it.request())
                } else {
                    throw NoNetworkException("No Network")
                }
            }
            .addInterceptor(VersionValidationInterceptor(appVersionManager))
            .addInterceptor(
                HttpLoggingInterceptor { Timber.tag("OkHttp").d("RetroLogger: $it") }
                    .setLevel(INTERCEPT_LOG_LEVEL)
            )
            .addInterceptor {
                it.proceed(
                    it.request()
                        .newBuilder()
                        .header("Content-Type", "application/json")
                        .build())
            }
            .build()


//    @Provides
//    @Singleton
//    @Named("bigantoApi")
//    fun provideRetrofitBigantoApi(context: Application): Retrofit =
//        Retrofit.Builder()
//            .baseUrl(IBigantoMobileApi.BASE_URL)
//            .client(client(AppVersionManager(context),LiveNetworkMonitor(context)))
//            .addConverterFactory(NullOnEmptyConverterFactory.create())
//            .addConverterFactory(gsonConverterFactory())
//            .addCallAdapterFactory(RxJava2CallAdapterFactory.create())
//            .build()

    //@Inject
    //lateinit var context: RoomParkApplication

    @Provides
    @Singleton
//    @Named("roomParkApi")
    fun provideRetrofitRoomParkApi(context: Application): Retrofit =
        Retrofit.Builder()
            .baseUrl(IRoomParkMobileApi.BASE_URL)
            .client(client(AppVersionManager(context), LiveNetworkMonitor(context)))
            .addConverterFactory(ScalarsConverterFactory.create())
            .addConverterFactory(NullOnEmptyConverterFactory.create())
            .addConverterFactory(gsonConverterFactory())
            .addCallAdapterFactory(RxJava2CallAdapterFactory.create())
            .build()

}